%
%
%  Track Plot Distance Between Tracks Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Tracks Functions">
%        <Item name="Plot Distance Between Tracks" icon="Matlab" tooltip="Plot distance between tracks.">
%          <Command>MatlabXT::XTTrackPlotDistanceBetweenTracks(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Plot Distance Between Tracks" icon="Matlab" tooltip="Plot distance between tracks.">
%            <Command>MatlabXT::XTTrackPlotDistanceBetweenTracks(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Plot Distance Between Tracks" icon="Matlab" tooltip="Plot distance between tracks.">
%            <Command>MatlabXT::XTTrackPlotDistanceBetweenTracks(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   
%   Plot the absolute value and the X,Y,Z components of the distance 
%       between the selected track and the other visible tracks that are 
%       in the same group.
%
%

function XTTrackPlotDistanceBetweenTracks(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some tracks
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some tracks in the surpass scene!');
  return
end

% get the track
vObjectIndices = GetFirstSelectedTrack(vImarisApplication);
if isempty(vObjectIndices)  
  msgbox('Please select a track!');
  return
end

vProgressDisplay = waitbar(0, 'Calculating distance');

[vCoords1, vTimes1] = GetObjectsCoordsTimes( ...
  vImarisApplication, vObjectIndices);

vObjects = vImarisApplication.GetSurpassSelection;
vFactory = vImarisApplication.GetFactory;
if vFactory.IsSpots(vObjects)
  vObjects = vFactory.ToSpots(vObjects);
else
  vObjects = vFactory.ToSurfaces(vObjects);
end
vEdges = vObjects.GetTrackEdges;
vTracksIndices = SplitEdgesListIntoTracks(vEdges);
vNumberOfTracks = numel(vTracksIndices);

% get the jet colormap
vColorMap = jet;
vColorMapSize = size(vColorMap,1);

for vPlot = 1:4
  subplot(2,2,vPlot)
  newplot
  hold on
end

vNumberOfTracksDisplayed = 0;
vTrackNames = cell(vNumberOfTracks, 1);

vObjectsTime = vTimes1;
vObjects1Index = vTimes1;
vObjects2Index = vTimes1;

vTrack1Name = '';

for vTrackIndex = 1:vNumberOfTracks
  vTrack2 = vTracksIndices{vTrackIndex};
  if vTrack2(1) == vObjectIndices(1)
    vTrack1Name = sprintf('Track %d', vTrackIndex);
    continue
  end

  [vCoords2, vTimes2] = GetObjectsCoordsTimes( ...
    vImarisApplication, vTrack2);

  % find the time points where both tracks are defined
  vNumberOfTimes = 0;
  for vIndex1 = 1:numel(vTimes1);
    vTime = vTimes1(vIndex1);
    vIndex2 = find(vTimes2 == vTime, 1);
    if ~isempty(vIndex2)
      vNumberOfTimes = vNumberOfTimes + 1;
      vObjectsTime(vNumberOfTimes) = vTime + 1;
      vObjects1Index(vNumberOfTimes) = vIndex1;
      vObjects2Index(vNumberOfTimes) = vIndex2;
    end
  end

  if vNumberOfTimes == 0
    % the two tracks are not commonly defined on any time point
    continue
  end
  
  % resize to valid data
  vIndex1 = vObjects1Index(1:vNumberOfTimes);
  vIndex2 = vObjects2Index(1:vNumberOfTimes);
  vCommonTimes = vObjectsTime(1:vNumberOfTimes);

  % distances are: X, Y, Z, norm(XYZ)
  vDistances = zeros(vNumberOfTimes, 4);
  for vIndex = 1:3
    vDistances(:, vIndex+1) = vCoords2(vIndex2, vIndex) - ...
      vCoords1(vIndex1, vIndex);
  end
  vDistances(:, 1) = sqrt(vDistances(:, 1).^2 + vDistances(:, 2).^2 + ...
    vDistances(:, 3).^2);

  % computes the color
  vPercent = (vTrackIndex-1)/max(vNumberOfTracks-1,1)*(vColorMapSize-1)+1;
  vFloor = floor(vPercent);
  vColor = vColorMap(vFloor,:);
  if vFloor<vPercent
    vPercent = vPercent - vFloor;
    vColor = vPercent*vColor+(1-vPercent)*vColorMap(vFloor+1,:);
  end

  for vPlot = 1:4
    subplot(2,2,vPlot)
    plot(vCommonTimes, vDistances(:, vPlot), 'Color', vColor);
  end

  % prepare the legend
  vNumberOfTracksDisplayed = vNumberOfTracksDisplayed + 1;
  vTrackNames{vNumberOfTracksDisplayed} = sprintf('Track %d', vTrackIndex);
  
  waitbar(vTrackIndex/vNumberOfTracks, vProgressDisplay)
end

vTrackNames = vTrackNames(1:vNumberOfTracksDisplayed);

vStrings = {'Absolute ','X-','Y-','Z-'};
for vPlot = 1:4
  subplot(2,2,vPlot)
  title([vStrings{vPlot}, 'distance from ', vTrack1Name]);
  legend(vTrackNames);
  xlabel('Time');
  ylabel('Distance [um]');
  hold off
end

close(vProgressDisplay);


%---------------------------------------------------------%

function aObjectIndices = GetFirstSelectedTrack(aImarisApplication)

% aTracks is the list of indices (starting from 0) of the objects
%   in the track, *not* a set of edges
aObjectIndices = [];

% get the selected object (spots or surfaces)
vFactory = aImarisApplication.GetFactory;
vObject = vFactory.ToSpots(aImarisApplication.GetSurpassSelection);
if ~isempty(vObject)
  vSize = numel(vObject.GetIndicesT);
else
  vObject = vFactory.ToSurfaces(aImarisApplication.GetSurpassSelection);
  if ~isempty(vObject)
    vSize = vObject.GetNumberOfSurfaces;
  else
    return
  end
end

% get the sub-selection and the traks
vSelection = vObject.GetSelectedIndices;
vEdges = vObject.GetTrackEdges';
if isempty(vSelection) || isempty(vEdges)
  return
end

% get the first selected spot which belongs to a of the track
vCurrent = [];
vIndex = 0;
while ~any(vCurrent) && vIndex < numel(vSelection)
  vIndex = vIndex + 1;
  vCurrent = any(vEdges == vSelection(vIndex));
end
if ~any(vCurrent(:))
  % no spots selected belongs to a track
  return
end

% get the spots of the track of the first selected object
vTrackBool = false(vSize, 1);
vTrackBool(vSelection(vIndex) + 1) = true;
while any(vCurrent)
  vObjects = vEdges([vCurrent; vCurrent]);
  vTrackBool(vObjects + 1) = true;
  vEdges([vCurrent; vCurrent]) = vSize;
  vCurrent = false(size(vCurrent));
  for vIndex = 1:numel(vObjects)
    vCurrent = vCurrent | any(vEdges == vObjects(vIndex));
  end
end
vIndices = 0:vSize-1;
aObjectIndices = vIndices(vTrackBool);


%---------------------------------------------------------%

function [aCoords, aTimes] = GetObjectsCoordsTimes( ...
  aImarisApplication, aSelectionIndices)

aCoords = [];
aTimes = [];

% get the selected object (spots or surfaces)
vFactory = aImarisApplication.GetFactory;
vSpots = vFactory.ToSpots(aImarisApplication.GetSurpassSelection);
vSurfaces = vFactory.ToSurfaces(aImarisApplication.GetSurpassSelection);

if ~isempty(vSpots)
  aCoords = vSpots.GetPositionsXYZ;
  aTimes = vSpots.GetIndicesT;
  vSize = numel(aTimes);
  % remove invalid selection indices
  aSelectionIndices = aSelectionIndices(aSelectionIndices < vSize) + 1;
  % perform selection
  aCoords = aCoords(aSelectionIndices, :);
  aTimes = aTimes(aSelectionIndices);
elseif ~isempty(vSurfaces)
  vSize = vSurfaces.GetNumberOfSurfaces;
  % remove invalid selection indices
  aSelectionIndices = aSelectionIndices(aSelectionIndices < vSize);
  vSize = numel(aSelectionIndices);
  aCoords = zeros(vSize, 3);
  aTimes = zeros(vSize, 1);
  % read selection
  for vIndex = 1:vSize
    vSurface = aSelectionIndices(vIndex);
    aCoords(vIndex, :) = vSurfaces.GetCenterOfMass(vSurface);
    aTimes(vIndex) = vSurfaces.GetTimeIndex(vSurface);
  end
end


%---------------------------------------------------------%

function aTracksIndices = SplitEdgesListIntoTracks(aEdges)

% compute the labels (one label for each connected set of objects)
vNumberOfObjects = max(aEdges(:)) + 1;
vLabels = zeros(vNumberOfObjects, 1);
vEdgeLabels = 1:size(aEdges, 1);

for vIndex = 1:size(aEdges, 1)
  vEdge = aEdges(vIndex, :) + 1;
  vLabel = vLabels(vEdge);
  vLabel = vLabel(vLabel > 0);
  
  if isempty(vLabel) 
    % no label yet -> create new
    vLabel = max(vLabels) + 1;
  elseif numel(vLabel) > 1 
    % multiple labels assigned to connected objects -> merge
    vFirst = min(vLabel);
    for vLabIndex = 1:numel(vLabel)
      vLab = vLabel(vLabIndex);
      if vLab ~= vFirst
        vLabels(vLabels == vLab) = vFirst;
        vLabels(vLabels > vLab) = vLabels(vLabels > vLab) - 1;
        vEdgeLabels(vEdgeLabels == vLab) = vFirst;
        vEdgeLabels(vEdgeLabels > vLab) = vEdgeLabels(vEdgeLabels > vLab) - 1;
      end
    end
    vLabel = vFirst;
  end
  vLabels(vEdge) = vLabel;
  vEdgeLabels(vIndex) = vLabel;
end

vNumTracks = max(vLabels);
aTracksIndices = cell(vNumTracks, 1);
vIndices = 0:vNumberOfObjects - 1;
for vIndex = 1:vNumTracks
  aTracksIndices{vIndex} = vIndices(vLabels == vIndex);
end
